const items = [
  { id: 0, title: 'Root item' },
  { id: 1, superIds: [ 0 ], priorId: 2, title: 'Second subitem' },
  { id: 2, superIds: [ 0, 3 ], title: 'First subitem' },
  { id: 3, superIds: [ 1, 2 ], title: 'Shared subitem of 1 & 2' },
  { id: 4, superIds: [ 1 ], title: 'First subitem of first subitem' },
  { id: 5, superIds: [ 1 ], priorId: 6, title: 'Third subitem of first subitem' },
  { id: 6, superIds: [ 1 ], priorId: 4, title: 'Second subitem of first subitem' },
  { id: 7, superIds: [ 2 ], title: 'First subitem of the second subitem' },
  { id: 8, superIds: [ 3 ], title: 'Subitem of the shared subitem' }
];

const ownerIds = {};

function render() {
  document.body.innerHTML = '';

  const dataDiv = document.createElement('div');
  dataDiv.classList.add('data');
  dataDiv.textContent = JSON.stringify(ownerIds);
  document.body.appendChild(dataDiv);

  for (const item of recurse(items, ownerIds)) {
    const itemDiv = document.createElement('div');
    itemDiv.classList.add('item');
    itemDiv.style.paddingLeft = `${item.level}em`;

    switch (item.state) {
      case 'expandable-implicitly-expanded':
      case 'expandable-explicitly-expanded': {
        const collapseButton = document.createElement('button');
        collapseButton.classList.add('directory-open');
        collapseButton.textContent = '-';

        collapseButton.addEventListener('click', _ => {
          items.find(i => i.id === item.item.id).state = 'collapsed';
          render();
        });

        itemDiv.appendChild(collapseButton);
        break;
      }

      case 'expandable-implicitly-collapsed': {
        const expandButton = document.createElement('button');
        expandButton.classList.add('directory-closed');
        expandButton.textContent = '+';

        expandButton.addEventListener('click', _ => {
          ownerIds[item.item.id] = item.superId;
          render();
        });

        itemDiv.appendChild(expandButton);
        break;
      }

      case 'expandable-explicitly-collapsed': {
        const expandButton = document.createElement('button');
        expandButton.classList.add('directory-closed');
        expandButton.textContent = '+';

        expandButton.addEventListener('click', _ => {
          items.find(i => i.id === item.item.id).state = 'expanded';
          render();
        });

        itemDiv.appendChild(expandButton);
        break;
      }

      case 'unexpandable': {
        const openButton = document.createElement('button');
        openButton.classList.add('file');
        openButton.textContent = '>';

        openButton.addEventListener('click', _ => {
          alert('TODO: Activate');
        });

        itemDiv.appendChild(openButton);
        break;
      }

      default: {
        throw new Error(`Invalid item state '${item.state}'.`);
      }
    }

    const titleSpan = document.createElement('span');
    titleSpan.classList.add('title');
    titleSpan.textContent = item.item.title;

    itemDiv.appendChild(titleSpan);

    const infoSpan = document.createElement('span');
    infoSpan.classList.add('info');
    infoSpan.textContent = `id: ${item.item.id} | superId: ${item.superId} | superIds: ${item.item.superIds} | priorId: ${item.item.priorId} | ${item.state}`;

    itemDiv.appendChild(infoSpan);

    document.body.appendChild(itemDiv);
  }
}

window.addEventListener('load', _ => {
  render();
});
