// TODO: Also indicate if item has subitems or not with a new state `unexpandable`
// - Adapt the other states to `expandable-implicitly/explicitly-expanded/collapsed`
function* recurse(items, ownerIds, superId = undefined, level = 0) {
  const subitems = [ ...getSubitems(items, superId) ];
  for (const [ item, index ] of getOrderedItems(subitems)) {
    // Find out underneath which `superId` the item should be expanded if it has multiple `superIds` (can only be expanded for one at a time)
    const ownerId = ownerIds[item.id];

    // Determine if item has subitems separately from walking subitems so we don't need to backup the stream by caching the nested `recurse`s
    const expandable = hasSubitems(items, item.id);

    // See if item is:
    // - explicitly expanded (`expanded`, user clicked +)
    // - explicitly collapsed (`collapsed`, user clicked -)
    // - implicitly expanded (`undefined` and current `superId` is `ownerId` if any or `superId` is first in `superIds` as a fallback)
    // - implicitly collapsed (`undefined` and current `superId` is not `ownerId` if any or `superId` is not first in `superIds` otherwise)
    switch (item.state) {
      case undefined:
      case 'expanded': {
        const expressed = item.state === undefined ? 'implicitly' : 'explicitly';
        if ((ownerId !== undefined) ? (superId === ownerId) : (index === 0)) {
          /* Either item has `ownerId` and it matches current `superId` or it doesn't and current `superId` is first in item's `superIds` */
          yield { item, level, state: expandable ? `expandable-${expressed}-expanded` : 'unexpandable', superId };
          if (expandable) {
            yield* recurse(items, ownerIds, item.id, level + 1);
          }
        } else {
          /* Either item has `ownerId` and it doesn't match current `superId` or it doesn't and current `superId` is not first in item's `superIds` */
          yield { item, level, state: expandable ? `expandable-${expressed}-collapsed` : 'unexpandable', superId };
        }

        break;
      }

      case 'collapsed': {
        yield { item, level, state: expandable ? `expandable-explicitly-collapsed` : 'unexpandable', superId };
        break;
      }

      default: {
        throw new Error(`Invalid item state '${item.state}'.`);
      }
    }
  }
}

// Determines of a given `itemId` has subitems (faster than `getSubitems`)
function hasSubitems(items, itemId) {
  return !!items.find(item => item.superIds && item.superIds.includes(itemId));
}

// Streams items in the order of given `priorId`
function* getOrderedItems(items, priorId = undefined) {
  for (const [ item, index ] of items) {
    if (item.priorId === priorId) {
      yield [ item, index ];
      yield* getOrderedItems(items, item.id);
    }
  }
}

// Streams items that are subitems of a given `superId`
function* getSubitems(items, superId) {
  for (const item of items) {
    // Fallback to `[ undefined ]` if `item` has no `superIds` so that `superId` `undefined` matches `index` `0` for root items
    const superIds = (item.superIds === undefined || item.superIds === null || item.superIds.length === 0) ? [ undefined ] : item.superIds;
    // Find the index of `superId` in `item`'s `superIds` to see if item is a subitem of `superId` and if it matches `ownerId` if any
    const index = superIds.indexOf(superId);

    if (~index) {
      yield [ item, index ];
    }
  }
}
